function [ph,tn,f]=MaxFlatDiffFourCycle(x,fs);
% Input x time series at sample frequency fs Hz
% Output phasor ph on time Tn
%SET PARAMETERS------------------------------------------------------------
%fs = 2400;                          % sampling frequency
Ts = 1/fs;                          % sampling cycle
f1 = 50;                            % fundamental frequency
T1 = 1/f1;
N1 = fs/f1;                         % data window = 1 cycle
N = 4*N1+1;
Nh = (N-1)/2;
w1 = 2*pi/N1;
kappa = 3;                          % set Taylor order
Tend=length(abs(x))*Ts;
%--------------------------------------------------------------------------



%MAIN PROGRAM BODY---------------------------------------------------------
b=[];
temp=[];
p=[];
deltaf=[];

%-----------------------------------left matrix
for k=0:kappa
    for l=1:(Nh+1)
        temp(l) = ((-(Nh+1-l))^(kappa-k))*exp((Nh+1-l)*w1*1i);
        if l~=(N-Nh)
            temp(2*Nh+2-l) = (((Nh+1-l))^(kappa-k))*exp(-(Nh+1-l)*w1*1i);
        end
    end
    b=[b,temp.'];           % forming one column, from the left to the right
    temp=[];
end
    
%-----------------------------------right matrix
for k=0:kappa
    for l=1:(Nh+1)
        temp(l) = ((-(Nh+1-l))^k)*exp(-(Nh+1-l)*w1*1i);
        if l~=(N-Nh)
            temp(2*Nh+2-l) = (((Nh+1-l))^k)*exp((Nh+1-l)*w1*1i);
        end
    end
    b=[b,temp.'];
    temp=[];
    
end
   
    W = diag(kaiser(N,8));                  % Kaiser window for weighting
    B = ((b')*(W')*W*b)\((b')*(W')*W);
    
    
%----------------------------phasor estimation-----------------------------

m=1;
for k=2*N1+1:N1:length(x)-3*N1
    
    p = 2*B*(x(k-Nh:k+Nh)');                % implementation of equation 10
    p = p(kappa+2:length(p));               % filter conjugated elements
        
    p(2) = p(2)*N1*f1;                      % calculate phasor's derivatives
    p(3) = p(3)*2*(N1*f1)^2;
        
    fdelta = imag(p(2)/p(1))/2/pi;                  % freq deviation
    fdev = imag(p(3)/p(1) - (p(2)/p(1))^2)/2/pi;    % freq's derivative
    
    phasor(m) = p(1);               % storing estimated phasor
    fest(m) = f1 + fdelta;          % freq estimation
    t(m) = (m+1)*T1;
    m = m+1;                        % just an index
end

% END OF MAIN PROGRAM BODY-------------------------------------------------

ph=phasor;   
tn=t;
f=fest;
% disp(errmax);
figure(1);
plot(tn,angle(phasor)*180/pi);
figure(2);
plot(tn,abs(phasor));













